@@ -1,27 +1,5 @@
-############################################################################
-#
 # Copyright (C) 2016 The Qt Company Ltd.
-# Contact: https://www.qt.io/licensing/
-#
-# This file is part of Qt Creator.
-#
-# Commercial License Usage
-# Licensees holding valid commercial Qt licenses may use this file in
-# accordance with the commercial license agreement provided with the
-# Software or, alternatively, in accordance with the terms contained in
-# a written agreement between you and The Qt Company. For licensing terms
-# and conditions see https://www.qt.io/terms-conditions. For further
-# information use the contact form at https://www.qt.io/contact-us.
-#
-# GNU General Public License Usage
-# Alternatively, this file may be used under the terms of the GNU
-# General Public License version 3 as published by the Free Software
-# Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-# included in the packaging of this file. Please review the following
-# information to ensure the GNU General Public License requirements will
-# be met: https://www.gnu.org/licenses/gpl-3.0.html.
-#
-############################################################################
+# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 import platform;
 import re;
@@ -34,7 +12,11 @@ import subprocess;
 import sys
 import errno;
 from datetime import datetime,timedelta;
-import __builtin__
+if sys.version_info.major > 2:
+    import builtins as __builtin__
+else:
+    import __builtin__
+
 
 srcPath = ''
 SettingsPath = []
@@ -54,9 +36,23 @@ source("../../shared/clang.py")
 source("../../shared/welcome.py")
 source("../../shared/workarounds.py") # include this at last
 
+settingsPathsWithExplicitlyEnabledClangd = set()
+
+def __closeInfoBarEntry__(leftButtonText):
+    toolButton = ("text='%s' type='QToolButton' unnamed='1' visible='1' "
+                  "window=':Qt Creator_Core::Internal::MainWindow'")
+    doNotShowAgain = toolButton % "Do Not Show Again"
+    leftWidget = "leftWidget={%s}" % (toolButton % leftButtonText)
+    test.log("closing %s" % leftButtonText)
+    buttonObjStr = "{%s %s}" % (doNotShowAgain, leftWidget)
+    checkIfObjectExists(buttonObjStr, verboseOnFail=True)
+    clickButton(waitForObject(buttonObjStr))
+    checkIfObjectExists(buttonObjStr, False, 2000, True)
+
 # additionalParameters must be a list or tuple of strings or None
 def startQC(additionalParameters=None, withPreparedSettingsPath=True, closeLinkToQt=True, cancelTour=True):
     global SettingsPath
+    global settingsPathsWithExplicitlyEnabledClangd
     appWithOptions = ['"Qt Creator"' if platform.system() == 'Darwin' else "qtcreator"]
     if withPreparedSettingsPath:
         appWithOptions.extend(SettingsPath)
@@ -66,12 +62,26 @@ def startQC(additionalParameters=None, withPreparedSettingsPath=True, closeLinkT
         appWithOptions.extend(('-platform', 'windows:dialogs=none'))
     test.log("Starting now: %s" % ' '.join(appWithOptions))
     appContext = startApplication(' '.join(appWithOptions))
-    if closeLinkToQt or cancelTour:
+    if (closeLinkToQt or cancelTour or
+        str(SettingsPath) not in settingsPathsWithExplicitlyEnabledClangd):
         progressBarWait(3000)  # wait for the "Updating documentation" progress bar
+    if str(SettingsPath) not in settingsPathsWithExplicitlyEnabledClangd:
+        # This block will incorrectly be skipped when a test calls startQC multiple times in a row
+        # passing different settings paths in "additionalParameters". Currently we don't have such
+        # a test. Even if we did, it would only make a difference if the test relied on clangd
+        # being active and ran on a machine with insufficient memory.
+        try:
+            mouseClick(waitForObject("{text='Enable Anyway' type='QToolButton' "
+                                     "unnamed='1' visible='1' "
+                                     "window=':Qt Creator_Core::Internal::MainWindow'}", 500))
+            settingsPathsWithExplicitlyEnabledClangd.add(str(SettingsPath))
+        except:
+            pass
+    if closeLinkToQt or cancelTour:
         if closeLinkToQt:
-            clickButton(waitForObject(":*Qt Creator.Do Not Show Again_QToolButton"))
+            __closeInfoBarEntry__("Link with Qt")
         if cancelTour:
-            clickButton(waitForObject(":*Qt Creator.Do Not Show Again_QToolButton"))
+            __closeInfoBarEntry__("Take UI Tour")
     return appContext;
 
 def startedWithoutPluginError():
@@ -94,7 +104,7 @@ def waitForCleanShutdown(timeOut=10):
     shutdownDone = (str(appCtxt)=="")
     if platform.system() in ('Windows','Microsoft'):
         # cleaning helper for running on the build machines
-        checkForStillRunningQmlExecutable(['qmlviewer.exe', 'qmlscene.exe'])
+        checkForStillRunningQmlExecutable(['qmlscene.exe', 'qml.exe'])
         endtime = datetime.utcnow() + timedelta(seconds=timeOut)
         while not shutdownDone:
             # following work-around because os.kill() works for win not until python 2.7
@@ -112,22 +122,11 @@ def waitForCleanShutdown(timeOut=10):
         while not shutdownDone:
             try:
                 os.kill(appCtxt.pid,0)
-            except OSError, err:
+            except OSError as err:
                 if err.errno == errno.EPERM or err.errno == errno.ESRCH:
                     shutdownDone=True
             if not shutdownDone and datetime.utcnow() > endtime:
                 break
-        if platform.system() == 'Linux' and JIRA.isBugStillOpen(15749):
-            pgrepOutput = getOutputFromCmdline(["pgrep", "-f", "qtcreator_process_stub"],
-                                               acceptedError=1)
-            pids = pgrepOutput.splitlines()
-            if len(pids):
-                print("Killing %d qtcreator_process_stub instances" % len(pids))
-            for pid in pids:
-                try:
-                    os.kill(__builtin__.int(pid), 9)
-                except OSError: # we might kill the parent before the current pid
-                    pass
 
 def checkForStillRunningQmlExecutable(possibleNames):
     for qmlHelper in possibleNames:
@@ -136,9 +135,10 @@ def checkForStillRunningQmlExecutable(possibleNames):
             continue
         else:
             if subprocess.call(["taskkill", "/F", "/FI", "IMAGENAME eq %s" % qmlHelper]) == 0:
-                print "Killed still running %s" % qmlHelper
+                print("Killed still running %s" % qmlHelper)
             else:
-                print "%s is still running - failed to kill it" % qmlHelper
+                print("%s is still running - failed to kill it" % qmlHelper)
+
 
 def __removeTestingDir__():
     def __removeIt__(directory):
@@ -169,15 +169,30 @@ def substituteTildeWithinToolchains(settingsDir):
     __substitute__(toolchains, "~", home)
     test.log("Substituted all tildes with '%s' inside toolchains.xml..." % home)
 
+
+def substituteTildeWithinQtVersion(settingsDir):
+    toolchains = os.path.join(settingsDir, "QtProject", 'qtcreator', 'qtversion.xml')
+    home = os.path.expanduser("~")
+    __substitute__(toolchains, "~", home)
+    test.log("Substituted all tildes with '%s' inside qtversion.xml..." % home)
+
+
+def substituteOnlineInstallerPath(settingsDir):
+    qtversions = os.path.join(settingsDir, "QtProject", 'qtcreator', 'qtversion.xml')
+    dflt = "C:/Qt" if platform.system() in ('Microsoft', 'Windows') else os.path.expanduser("~/Qt")
+    replacement = str(os.getenv("SYSTEST_QTOI_BASEPATH", dflt)).replace('\\', '/')
+    while replacement.endswith('/'):
+        replacement = replacement[:-1]
+    __substitute__(qtversions, "SQUISH_QTOI_BASEPATH", replacement)
+    test.log("Substituted online installer base path (%s) inside qtversions.xml." % replacement)
+
+
 def substituteDefaultCompiler(settingsDir):
     compiler = None
     if platform.system() == 'Darwin':
         compiler = "clang_64"
     elif platform.system() == 'Linux':
-        if __is64BitOS__():
-            compiler = "gcc_64"
-        else:
-            compiler = "gcc"
+        compiler = "gcc_64"
     else:
         test.warning("Called substituteDefaultCompiler() on wrong platform.",
                      "This is a script error.")
@@ -187,93 +202,72 @@ def substituteDefaultCompiler(settingsDir):
         test.log("Injected default compiler '%s' to qtversion.xml..." % compiler)
 
 def substituteCdb(settingsDir):
-    def canUse32bitCdb():
+    def canUse64bitCdb():
         try:
             serverIni = readFile(os.path.join(os.getenv("APPDATA"), "froglogic",
                                               "Squish", "ver1", "server.ini"))
-            autLine = filter(lambda line: "AUT/qtcreator" in line, serverIni.splitlines())[0]
+            autLine = list(filter(lambda line: "AUT/qtcreator" in line, serverIni.splitlines()))[0]
             autPath = autLine.split("\"")[1]
-            return os.path.exists(os.path.join(autPath, "..", "lib", "qtcreatorcdbext32"))
+            return os.path.exists(os.path.join(autPath, "..", "lib", "qtcreatorcdbext64"))
         except:
             test.fatal("Something went wrong when determining debugger bitness",
                        "Did Squish's file structure change? Guessing 32-bit cdb can be used...")
             return True
 
-    if canUse32bitCdb():
-        architecture = "x86"
-        bitness = "32"
-    else:
+    if canUse64bitCdb():
         architecture = "x64"
         bitness = "64"
+    else:
+        architecture = "x86"
+        bitness = "32"
     debuggers = os.path.join(settingsDir, "QtProject", 'qtcreator', 'debuggers.xml')
     __substitute__(debuggers, "SQUISH_DEBUGGER_ARCHITECTURE", architecture)
     __substitute__(debuggers, "SQUISH_DEBUGGER_BITNESS", bitness)
     test.log("Injected architecture '%s' and bitness '%s' in cdb path..." % (architecture, bitness))
 
-def __guessABI__(supportedABIs, use64Bit):
-    if platform.system() == 'Linux':
-        supportedABIs = filter(lambda x: 'linux' in x, supportedABIs)
-    elif platform.system() == 'Darwin':
-        supportedABIs = filter(lambda x: 'darwin' in x, supportedABIs)
-    if use64Bit:
-        searchFor = "64bit"
-    else:
-        searchFor = "32bit"
-    for abi in supportedABIs:
-        if searchFor in abi:
-            return abi
-    if use64Bit:
-        test.log("Supported ABIs do not include an ABI supporting 64bit - trying 32bit now")
-        return __guessABI__(supportedABIs, False)
-    test.fatal('Could not guess ABI!',
-               'Given ABIs: %s' % str(supportedABIs))
-    return ''
 
-def __is64BitOS__():
-    if platform.system() in ('Microsoft', 'Windows'):
-        machine = os.getenv("PROCESSOR_ARCHITEW6432", os.getenv("PROCESSOR_ARCHITECTURE"))
-    else:
-        machine = platform.machine()
-    if machine:
-        return '64' in machine
-    else:
-        return False
+def substituteMsvcPaths(settingsDir, version, targetBitness=64):
+    if not version in ['2017', '2019']:
+        test.fatal('Unexpected MSVC version - "%s" not implemented yet.' % version)
+        return
 
-def substituteUnchosenTargetABIs(settingsDir):
-    class ReadState:
-        NONE = 0
-        READING = 1
-        CLOSED = 2
+    hostArch = "Hostx64" if targetBitness == 64 else "Hostx86"
+    targetArch = "x64" if targetBitness == 64 else "x86"
+    for msvcFlavor in ["Community", "BuildTools"]:
+        try:
+            msvcPath = os.path.join("C:\\Program Files (x86)", "Microsoft Visual Studio",
+                                    version, msvcFlavor, "VC", "Tools", "MSVC")
+            foundVersions = os.listdir(msvcPath) # undetermined order
+            foundVersions.sort(reverse=True) # we explicitly want the latest and greatest
+            msvcPath = os.path.join(msvcPath, foundVersions[0], "bin", hostArch, targetArch)
+            __substitute__(os.path.join(settingsDir, "QtProject", 'qtcreator', 'toolchains.xml'),
+                           "SQUISH_MSVC%s_%d_PATH" % (version, targetBitness), msvcPath)
+            return
+        except:
+            continue
+    test.warning("PATH variable for MSVC%s could not be set, some tests will fail." % version,
+                 "Please make sure that MSVC%s is installed correctly." % version)
+
+
+def prependWindowsKit(settingsDir, targetBitness=64):
+    targetArch = "x64" if targetBitness == 64 else "x86"
+    profilesPath = os.path.join(settingsDir, 'QtProject', 'qtcreator', 'profiles.xml')
+    winkits = os.path.join("C:\\Program Files (x86)", "Windows Kits", "10")
+    if not os.path.exists(winkits):
+        __substitute__(profilesPath, "SQUISH_ENV_MODIFICATION", "")
+        return
+    possibleVersions = os.listdir(os.path.join(winkits, 'bin'))
+    possibleVersions.reverse() # prefer higher versions
+    for version in possibleVersions:
+        if not version.startswith("10"):
+            continue
+        toolsPath = os.path.join(winkits, 'bin', version, targetArch)
+        if os.path.exists(os.path.join(toolsPath, 'rc.exe')):
+            __substitute__(profilesPath, "SQUISH_ENV_MODIFICATION", "PATH=+%s" % toolsPath)
+            return
+    test.warning("Windows Kit path could not be added, some tests mail fail.")
+    __substitute__(profilesPath, "SQUISH_ENV_MODIFICATION", "")
 
-    on64Bit = __is64BitOS__()
-    toolchains = os.path.join(settingsDir, "QtProject", 'qtcreator', 'toolchains.xml')
-    origToolchains = toolchains + "_orig"
-    os.rename(toolchains, origToolchains)
-    origFile = open(origToolchains, "r")
-    modifiedFile = open(toolchains, "w")
-    supported = []
-    readState = ReadState.NONE
-    for line in origFile:
-        if readState == ReadState.NONE:
-            if "SupportedAbis" in line:
-                supported = []
-                readState = ReadState.READING
-        elif readState == ReadState.READING:
-            if "</valuelist>" in line:
-                readState = ReadState.CLOSED
-            else:
-                supported.append(line.split(">", 1)[1].rsplit("<", 1)[0])
-        elif readState == ReadState.CLOSED:
-            if "SupportedAbis" in line:
-                supported = []
-                readState = ReadState.READING
-            elif "SET_BY_SQUISH" in line:
-                line = line.replace("SET_BY_SQUISH", __guessABI__(supported, on64Bit))
-        modifiedFile.write(line)
-    origFile.close()
-    modifiedFile.close()
-    os.remove(origToolchains)
-    test.log("Substituted unchosen ABIs inside toolchains.xml...")
 
 def copySettingsToTmpDir(destination=None, omitFiles=[]):
     global tmpSettingsDir, SettingsPath, origSettingsDir
@@ -300,18 +294,22 @@ def copySettingsToTmpDir(destination=None, omitFiles=[]):
                 shutil.copy(os.path.join(r, ff), currentPath)
     if platform.system() in ('Linux', 'Darwin'):
         substituteTildeWithinToolchains(tmpSettingsDir)
+        substituteTildeWithinQtVersion(tmpSettingsDir)
         substituteDefaultCompiler(tmpSettingsDir)
     elif platform.system() in ('Windows', 'Microsoft'):
         substituteCdb(tmpSettingsDir)
-    substituteUnchosenTargetABIs(tmpSettingsDir)
+        substituteMsvcPaths(tmpSettingsDir, '2017', 64)
+        substituteMsvcPaths(tmpSettingsDir, '2017', 32)
+        substituteMsvcPaths(tmpSettingsDir, '2019', 64)
+        prependWindowsKit(tmpSettingsDir, 32)
+    substituteOnlineInstallerPath(tmpSettingsDir)
     SettingsPath = ['-settingspath', '"%s"' % tmpSettingsDir]
 
+test.log("Test is running on Python %s" % sys.version)
 # current dir is directory holding qtcreator.py
 origSettingsDir = os.path.abspath(os.path.join(os.getcwd(), "..", "..", "settings"))
-qt4Path = os.path.expanduser("~/Qt4.8.7")
 
 if platform.system() in ('Windows', 'Microsoft'):
-    qt4Path = "C:\\Qt\\Qt4.8.7"
     origSettingsDir = os.path.join(origSettingsDir, "windows")
 elif platform.system() == 'Darwin':
     origSettingsDir = os.path.join(origSettingsDir, "mac")
